#-*- coding: utf-8 -*-
#
# Script_Graphe_10.py
#
# https://marcarea.com/weblog/2019/02/17/parcours-de-graphes-en-python
# https://docs.python.org/fr/3/library/collections.html#collections.deque


from collections import deque


class Graphe:
    """un graphe représenté par une matrice d'adjacence,
       où les sommets sont les entiers 0,1,...,n-1"""

    def __init__(self, n):
        self.n = n
        self.adj = [[False] * n for _ in range(n)]

    def ajouter_arc(self, s1, s2):
        self.adj[s1][s2] = True

    def ajouter_arete(self, s1, s2):
        self.adj[s1][s2] = True
        self.adj[s2][s1] = True
        
    def arc(self, s1, s2):
        return self.adj[s1][s2]

    def voisins(self, s):
        v = []
        for i in range(self.n):
            if self.adj[s][i]:
                v.append(i)
        return v

    def imprime_adj(self):
        '''
        Impression de la matrice d'adjacence
        '''



    def __str__(self):
        '''
        Impression de la matrice d'adjacence par print(g)
        Où g est une instantiation de la Classe Graphe
        '''
 

def iterative_dfs(graph, node):
    '''
    Parcours en profondeur DFS
    Implémentation itérative.
    Utilisation d'une Pile (stack en anglais) les successeurs du noeud en 
    cours d'analyse sont insérés dans la pile. La pile se vide au fur et à 
    mesure de la boucle non bornée while.
    L'implantation de la pile repose sur le module Python collections.deque
    Au fur et à mesure de leur découverte les noeuds sont ajoutés à la liste
    visited qui est renvoyée en résultat 
    '''
    # Initialisations
    visited = []
    stack = deque()
    stack.append(node)

    # Tant que la Pile n'est pas vide
    while stack:
        # On extrait l'élément de droite de la pile
        node = stack.pop()
        # Si le noeud extrait n'est pas déjà visité
        if node not in visited:
            # On l'ajoute à la liste des noeuds visités
            visited.append(node)
            # On établi la liste des noeuds fils non encore
            # visités
            unvisited = [n for n in g.voisins(node)  if n not in visited]
            # On insère cette liste dans la Pile
            stack.extend(unvisited)

    # Retour du résultat    
    return visited



# Complétez la description du graphe
g = Graphe(15)
g.ajouter_arete(0,1)
g.ajouter_arete(0,2)


print("\n"*30)

print("PARCOURS DE GRAPHE EN PROFONDEUR")
print("================================")
sommet_de_depart = input("Quel est le sommet de départ (0 par défaut) : ")
if sommet_de_depart == "":
    sommet_de_depart = 0
else:
    sommet_de_depart = int(sommet_de_depart)
    

print()
print()
print("Parcours DFS itératif ",end="")
print()
print("Sommet de départ : ",sommet_de_depart)
liste_dfs = iterative_dfs(g, sommet_de_depart)
print("Parcours ",liste_dfs)

