#-*- coding: utf-8 -*-
#
# Script_Graphe_9.py
#
# https://marcarea.com/weblog/2019/02/17/parcours-de-graphes-en-python
# https://docs.python.org/fr/3/library/collections.html#collections.deque


from collections import deque


class Graphe:
    """un graphe représenté par une matrice d'adjacence,
       où les sommets sont les entiers 0,1,...,n-1"""

    def __init__(self, n):
        self.n = n
        self.adj = [[False] * n for _ in range(n)]

    def ajouter_arc(self, s1, s2):
        self.adj[s1][s2] = True

    def ajouter_arete(self, s1, s2):
        self.adj[s1][s2] = True
        self.adj[s2][s1] = True
        
    def arc(self, s1, s2):
        return self.adj[s1][s2]

    def voisins(self, s):
        v = []
        for i in range(self.n):
            if self.adj[s][i]:
                v.append(i)
        return v

    def imprime_adj(self):
        '''
        Impression de la matrice d'adjacence
        '''



    def __str__(self):
        '''
        Impression de la matrice d'adjacence par print(g)
        Où g est une instantiation de la Classe Graphe
        '''


def recursive_dfs(graph, node, visited=None):
    '''
    Parcours en profondeur DFS
    Implémentation récursive
    Le fonctionnement est réalisé avec deux listes :
    visited   : contient la liste des noeuds déjà explorés
    unvisited : donne la liste des noeuds non encore visité à partir
                du noeud courant.
    '''
    # Initialisation de la liste des sommets
    # visités
    if visited is None:
        visited = []
        
    # Si le noeud courant n'est pas dans la liste
    # des noeuds visité on l'ajoute    
    if node not in visited:
        visited.append(node)
    
    # Liste des noeuds non encore visités à partir
    # du noeud courant    
    unvisited = [n for n in g.voisins(node) if n not in visited] 
    
    # On poursuit en profondeur pour tous les noeuds
    # restants à découvrir    
    for node in unvisited:
        recursive_dfs(graph, node, visited)
    
    # Retour du résultat    
    return visited        

# Complétez la description du graphe
g = Graphe(15)
g.ajouter_arete(0,1)
g.ajouter_arete(0,2)






print("\n"*30)

print("PARCOURS DE GRAPHE EN PROFONDEUR")
print("================================")
sommet_de_depart = input("Quel est le sommet de départ (0 par défaut) : ")
if sommet_de_depart == "":
    sommet_de_depart = 0
else:
    sommet_de_depart = int(sommet_de_depart)
    
print()
print("Parcours DFS récursif ",end="")
print()
print("Sommet de départ : ",sommet_de_depart)
liste_dfs = recursive_dfs(g, sommet_de_depart)
print("Parcours ",liste_dfs)

